<?php

namespace GeorgGriesser\WordPress\Velocious\Smart_WP;

class Header_Generator
{
    /**
     * @param $post
     * @param $mtime
     * @param $options
     *
     * @return array
     */
    static function batch_generate_headers($post, $mtime, $options)
    {
        $headers_arr = array();

        if ($options['add_etag_header'] == true) {
            $headers_arr['ETag'] = self::generate_etag_header($post, $mtime, $options);
        }

        if ($options['add_last_modified_header'] == true) {
            $headers_arr['Last-Modified'] = self::generate_last_modified_header($post, $mtime, $options);
        }

        if ($options['add_cache_control_header'] == true) {
            $headers_arr['Cache-Control'] = self::generate_cache_control_header($post, $mtime, $options);
        }

        $headers_arr = apply_filters('smart_wp_headers', $headers_arr);

        return $headers_arr;
    }


    /**
     *
     * The generated ETag is unique for every post page. In order to generate it
     * ``$wp->query_vars`` are used among other properties so as to generate a
     * unique ETag even for archive on which the latest post is the same.*
     *
     * @param $post
     * @param $mtime
     * @param $options
     * @return string
     */
    private static function generate_etag_header($post, $mtime, $options)
    {
        global $wp;

        $to_hash = array($mtime, $post->post_date_gmt, $post->guid, $post->ID, serialize($wp->query_vars));
        $header_etag_value = sha1(serialize($to_hash));
        // Generate a weak or strong ETag
        if ($options['generate_weak_etag'] == true) {
            return sprintf('W/"%s"', $header_etag_value);
        } else {
            return sprintf('"%s"', $header_etag_value);
        }

    }


    /**
     * @param $post
     * @param $mtime
     * @param $options
     * @return mixed
     */
    private static function generate_last_modified_header($post, $mtime, $options)
    {

        $header_last_modified_value = str_replace('+0000', 'GMT', gmdate('r', $mtime));
        return $header_last_modified_value;
    }


    /**
     * @param $post
     * @param $mtime
     * @param $options
     * @return string
     */
    private static function generate_cache_control_header($post, $mtime, $options)
    {

        // Adjust for authenticated users.
        if (is_user_logged_in()) {
            $options['cache_max_age_seconds'] = $options['cache_max_age_seconds_for_authenticated_users'];
        }

        // Adjust for search results.
        if (is_search()) {
            $options['cache_max_age_seconds'] = $options['cache_max_age_seconds_for_search_results'];
        }

        // Adjust for old content.
        if (self::is_old_content($mtime, $options)) {
            $options['cache_max_age_seconds'] = $options['cache_old_content_max_age_seconds'];
        }


        if (intval($options['cache_max_age_seconds']) > 0) {

            $default_cache_control_template = 'max-age=%s';
            $cache_control_template = apply_filters('addh_cache_control_header_format', $default_cache_control_template);

            $header_cache_control_value = sprintf($cache_control_template, $options['cache_max_age_seconds']);

            return $header_cache_control_value;
        } else {
            return 'no-cache, must-revalidate, max-age=0';
        }
    }

    /**
     * Checks if the content object has not been edited within the time defined als the 'cache_old_content_threshold'
     *
     * @param $modified_time
     * @param $options
     * @return bool
     */
    private static function is_old_content($modified_time, $options)
    {
        if (absint($options['cache_old_content_threshold_seconds']) <= 0) {

            return false;
        }

        $threshold_time_unix = absint(gmdate('U')) - absint($options['cache_old_content_threshold_seconds']);

        if ($modified_time > $threshold_time_unix) {

            return false;
        }

        return true;
    }
}